/*
 * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
 * @license AGPL-3.0
 *
 * This code is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
 *
 * Any use of the work other than as authorized under this license or copyright law is prohibited.
 *
 */
package com.openexchange.usm.datatypes.tasks.calendar;

import java.io.Serializable;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * Defines Participant Identifier in the Tasks Module. 
 *  
 * @author ldo
 *
 */
public final class ParticipantIdentifier implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	static final String TYPE = "type";
	static final String ID = "id";
	static final String DISPLAY_NAME = "display_name";
	static final String MAIL = "mail";
	/** User ID. */
	private final int _id;
	/** Type of participant:
	 * 1 	user
	 * 2 	user group
	 * 3 	resource
	 * 4 	resource group
	 * 5 	external user 
	*/
	private final int _type;

	private final String _displayName;

	private final String _mail;

	public ParticipantIdentifier(int id, int type, String displayName, String mail) {
		_id = id;
		_type = type;
		_displayName = displayName == null ? "" : displayName;
		_mail = mail == null ? "" : mail;
	}

	public ParticipantIdentifier(JSONObject jObject) throws JSONException {
		_type = jObject.getInt(TYPE);
		switch (_type) {
			case 1:
			case 2:
			case 4:
				_id = jObject.getInt(ID);
				_displayName = "";
				_mail = "";
				break;
            case 3:
                _id = jObject.getInt(ID);
                _displayName = jObject.has(DISPLAY_NAME) ? (jObject.isNull(DISPLAY_NAME) ? "" : jObject.getString(DISPLAY_NAME)) : "";
                _mail = jObject.has(MAIL) ?(jObject.isNull(MAIL) ? "" : jObject.getString(MAIL)) : "";
                break;
			case 5:
				_id = 0;
				_displayName = jObject.has(DISPLAY_NAME) ? (jObject.isNull(DISPLAY_NAME) ? "" : jObject.getString(DISPLAY_NAME)) : "";
				_mail = jObject.has(MAIL) ?(jObject.isNull(MAIL) ? "" : jObject.getString(MAIL)) : "";
				break;
			default:
				throw new JSONException("Invalid ParticipantIdentifier type " + _type);
		}
	}

	public ParticipantIdentifier() {
		_id = 0;
		_type = 0;
		_displayName = "";
		_mail = "";
	}

	public int getId() {
		return _id;
	}

	public int getType() {
		return _type;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + _id;
		result = prime * result + _type;
		result = prime * result + _displayName.hashCode();
		result = prime * result + _mail.hashCode();
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ParticipantIdentifier other = (ParticipantIdentifier) obj;
		if (_id != other.getId())
			return false;
		if (_type != other.getType())
			return false;
		if (!_displayName.equals(other.getDisplay_name()))
			return false;
		if (!_mail.equals(other.getMail()))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return '{' + ID + ':' + String.valueOf(_id) + ';' + TYPE + ':' + String.valueOf(_type) + ';' + DISPLAY_NAME
				+ ':' + _displayName + ';' + MAIL + ':' + _mail + '}';
	}

	public JSONObject toJSONObject() throws JSONException {
		JSONObject result = new JSONObject();
		result.put(ID, _id);
		result.put(TYPE, _type);
		if (_type == 5 || _type == 3) {
			result.put(DISPLAY_NAME, _displayName);
			result.put(MAIL, _mail);
		}
		return result;
	}

	public String getDisplay_name() {
		return _displayName;
	}

	public String getMail() {
		return _mail;
	}
}
